# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Receiver(resource.Resource):
    resource_key = 'receiver'
    resources_key = 'receivers'
    base_path = '/receivers'

    # Capabilities
    allow_list = True
    allow_fetch = True
    allow_create = True
    allow_commit = True
    allow_delete = True

    commit_method = 'PATCH'

    _query_mapping = resource.QueryParameters(
        'name', 'type', 'cluster_id', 'action', 'sort', 'global_project',
        user_id='user')

    # Properties
    #: The name of the receiver.
    name = resource.Body('name')
    #: The type of the receiver.
    type = resource.Body('type')
    #: The ID of the user who created the receiver, thus the owner of it.
    user_id = resource.Body('user')
    #: The ID of the project this receiver belongs to.
    project_id = resource.Body('project')
    #: The domain ID of the receiver.
    domain_id = resource.Body('domain')
    #: The ID of the targeted cluster.
    cluster_id = resource.Body('cluster_id')
    #: The name of the targeted action.
    action = resource.Body('action')
    #: Timestamp of when the receiver was created.
    created_at = resource.Body('created_at')
    #: Timestamp of when the receiver was last updated.
    updated_at = resource.Body('updated_at')
    #: The credential of the impersonated user.
    actor = resource.Body('actor', type=dict)
    #: A dictionary containing key-value pairs that are provided to the
    #: targeted action.
    params = resource.Body('params', type=dict)
    #: The information about the channel through which you can trigger the
    #: receiver hence the associated action.
    channel = resource.Body('channel', type=dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base


magnum_service_obj = dict(
    binary='fake-service',
    created_at='2015-08-27T09:49:58-05:00',
    disabled_reason=None,
    host='fake-host',
    human_id=None,
    id=1,
    report_count=1,
    state='up',
    updated_at=None,
)


class TestMagnumServices(base.TestCase):

    def test_list_magnum_services(self):
        self.register_uris([dict(
            method='GET',
            uri=self.get_mock_url(
                service_type='container-infrastructure-management',
                resource='mservices'),
            json=dict(mservices=[magnum_service_obj]))])
        mservices_list = self.cloud.list_magnum_services()
        self.assertEqual(
            mservices_list[0],
            self.cloud._normalize_magnum_service(magnum_service_obj))
        self.assert_calls()



# Copyright (c) 2018 China Telecom Corporation
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from openstack.network.v2 import firewall_rule

EXAMPLE = {
    'action': 'allow',
    'description': '1',
    'destination_ip_address': '10.0.0.2/24',
    'destination_port': '2',
    'name': '3',
    'enabled': True,
    'ip_version': 4,
    'protocol': 'tcp',
    'shared': True,
    'source_ip_address': '10.0.1.2/24',
    'source_port': '5',
    'project_id': '6',
}


class TestFirewallRule(testtools.TestCase):

    def test_basic(self):
        sot = firewall_rule.FirewallRule()
        self.assertEqual('firewall_rule', sot.resource_key)
        self.assertEqual('firewall_rules', sot.resources_key)
        self.assertEqual('/fwaas/firewall_rules', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = firewall_rule.FirewallRule(**EXAMPLE)
        self.assertEqual(EXAMPLE['action'], sot.action)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['destination_ip_address'],
                         sot.destination_ip_address)
        self.assertEqual(EXAMPLE['destination_port'], sot.destination_port)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['enabled'], sot.enabled)
        self.assertEqual(EXAMPLE['ip_version'], sot.ip_version)
        self.assertEqual(EXAMPLE['protocol'], sot.protocol)
        self.assertEqual(EXAMPLE['shared'], sot.shared)
        self.assertEqual(EXAMPLE['source_ip_address'],
                         sot.source_ip_address)
        self.assertEqual(EXAMPLE['source_port'], sot.source_port)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import pool
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'admin_state_up': True,
    'description': '2',
    'healthmonitor_id': '3-1',
    'health_monitors': ['3'],
    'health_monitor_status': ['4'],
    'id': IDENTIFIER,
    'lb_algorithm': '5',
    'listeners': [{'id': '6'}],
    'listener_id': '6',
    'members': [{'id': '7'}],
    'name': '8',
    'project_id': '9',
    'protocol': '10',
    'provider': '11',
    'session_persistence': '12',
    'status': '13',
    'status_description': '14',
    'subnet_id': '15',
    'loadbalancers': [{'id': '16'}],
    'loadbalancer_id': '16',
    'vip_id': '17',
}


class TestPool(base.TestCase):

    def test_basic(self):
        sot = pool.Pool()
        self.assertEqual('pool', sot.resource_key)
        self.assertEqual('pools', sot.resources_key)
        self.assertEqual('/lbaas/pools', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = pool.Pool(**EXAMPLE)
        self.assertTrue(sot.is_admin_state_up)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['healthmonitor_id'], sot.health_monitor_id)
        self.assertEqual(EXAMPLE['health_monitors'], sot.health_monitor_ids)
        self.assertEqual(EXAMPLE['health_monitor_status'],
                         sot.health_monitor_status)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['lb_algorithm'], sot.lb_algorithm)
        self.assertEqual(EXAMPLE['listeners'], sot.listener_ids)
        self.assertEqual(EXAMPLE['listener_id'], sot.listener_id)
        self.assertEqual(EXAMPLE['members'], sot.member_ids)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['protocol'], sot.protocol)
        self.assertEqual(EXAMPLE['provider'], sot.provider)
        self.assertEqual(EXAMPLE['session_persistence'],
                         sot.session_persistence)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['status_description'], sot.status_description)
        self.assertEqual(EXAMPLE['subnet_id'], sot.subnet_id)
        self.assertEqual(EXAMPLE['loadbalancers'], sot.load_balancer_ids)
        self.assertEqual(EXAMPLE['loadbalancer_id'], sot.load_balancer_id)
        self.assertEqual(EXAMPLE['vip_id'], sot.virtual_ip_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import network_ip_availability
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'network_id': IDENTIFIER,
    'network_name': 'private',
    'subnet_ip_availability': [],
    'project_id': '5',
    'total_ips': 6,
    'used_ips': 10,
}

EXAMPLE_WITH_OPTIONAL = {
    'network_id': IDENTIFIER,
    'network_name': 'private',
    'subnet_ip_availability': [{"used_ips": 3, "subnet_id":
                                "2e4db1d6-ab2d-4bb1-93bb-a003fdbc9b39",
                                "subnet_name": "private-subnet",
                                "ip_version": 6, "cidr": "fd91:c3ba:e818::/64",
                                "total_ips": 18446744073709551614}],
    'project_id': '2',
    'total_ips': 1844,
    'used_ips': 6,
}


class TestNetworkIPAvailability(base.TestCase):

    def test_basic(self):
        sot = network_ip_availability.NetworkIPAvailability()
        self.assertEqual('network_ip_availability', sot.resource_key)
        self.assertEqual('network_ip_availabilities', sot.resources_key)
        self.assertEqual('/network-ip-availabilities', sot.base_path)
        self.assertEqual('network_name', sot.name_attribute)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = network_ip_availability.NetworkIPAvailability(**EXAMPLE)
        self.assertEqual(EXAMPLE['network_id'], sot.network_id)
        self.assertEqual(EXAMPLE['network_name'], sot.network_name)
        self.assertEqual(EXAMPLE['subnet_ip_availability'],
                         sot.subnet_ip_availability)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['total_ips'], sot.total_ips)
        self.assertEqual(EXAMPLE['used_ips'], sot.used_ips)

    def test_make_it_with_optional(self):
        sot = network_ip_availability.NetworkIPAvailability(
            **EXAMPLE_WITH_OPTIONAL)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['network_id'], sot.network_id)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['network_name'],
                         sot.network_name)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['subnet_ip_availability'],
                         sot.subnet_ip_availability)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['total_ips'], sot.total_ips)
        self.assertEqual(EXAMPLE_WITH_OPTIONAL['used_ips'], sot.used_ips)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_volume
----------------------------------

Functional tests for `shade` block storage methods.
"""
import testtools

from openstack.cloud import exc
from openstack.tests.functional import base


class TestVolumeType(base.BaseFunctionalTest):

    def _assert_project(self, volume_name_or_id, project_id, allowed=True):
        acls = self.operator_cloud.get_volume_type_access(volume_name_or_id)
        allowed_projects = [x.get('project_id') for x in acls]
        self.assertEqual(allowed, project_id in allowed_projects)

    def setUp(self):
        super(TestVolumeType, self).setUp()
        if not self.user_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        volume_type = {
            "name": 'test-volume-type',
            "description": None,
            "os-volume-type-access:is_public": False}
        self.operator_cloud.block_storage.post(
            '/types', json={'volume_type': volume_type})

    def tearDown(self):
        ret = self.operator_cloud.get_volume_type('test-volume-type')
        if ret.get('id'):
            self.operator_cloud.block_storage.delete(
                '/types/{volume_type_id}'.format(volume_type_id=ret.id))
        super(TestVolumeType, self).tearDown()

    def test_list_volume_types(self):
        volume_types = self.operator_cloud.list_volume_types()
        self.assertTrue(volume_types)
        self.assertTrue(any(
            x for x in volume_types if x.name == 'test-volume-type'))

    def test_add_remove_volume_type_access(self):
        volume_type = self.operator_cloud.get_volume_type('test-volume-type')
        self.assertEqual('test-volume-type', volume_type.name)

        self.operator_cloud.add_volume_type_access(
            'test-volume-type',
            self.operator_cloud.current_project_id)
        self._assert_project(
            'test-volume-type', self.operator_cloud.current_project_id,
            allowed=True)

        self.operator_cloud.remove_volume_type_access(
            'test-volume-type',
            self.operator_cloud.current_project_id)
        self._assert_project(
            'test-volume-type', self.operator_cloud.current_project_id,
            allowed=False)

    def test_add_volume_type_access_missing_project(self):
        # Project id is not valitaded and it may not exist.
        self.operator_cloud.add_volume_type_access(
            'test-volume-type',
            '00000000000000000000000000000000')

        self.operator_cloud.remove_volume_type_access(
            'test-volume-type',
            '00000000000000000000000000000000')

    def test_add_volume_type_access_missing_volume(self):
        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "VolumeType not found.*"
        ):
            self.operator_cloud.add_volume_type_access(
                'MISSING_VOLUME_TYPE',
                self.operator_cloud.current_project_id)

    def test_remove_volume_type_access_missing_volume(self):
        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "VolumeType not found.*"
        ):
            self.operator_cloud.remove_volume_type_access(
                'MISSING_VOLUME_TYPE',
                self.operator_cloud.current_project_id)

    def test_add_volume_type_access_bad_project(self):
        with testtools.ExpectedException(
                exc.OpenStackCloudBadRequest,
                "Unable to authorize.*"
        ):
            self.operator_cloud.add_volume_type_access(
                'test-volume-type',
                'BAD_PROJECT_ID')

    def test_remove_volume_type_access_missing_project(self):
        with testtools.ExpectedException(
                exc.OpenStackCloudURINotFound,
                "Unable to revoke.*"
        ):
            self.operator_cloud.remove_volume_type_access(
                'test-volume-type',
                '00000000000000000000000000000000')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Workflow(resource.Resource):
    resource_key = 'workflow'
    resources_key = 'workflows'
    base_path = '/workflows'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'marker', 'limit', 'sort_keys', 'sort_dirs', 'fields')

    #: The name of this Workflow
    name = resource.Body("name")
    #: The inputs for this Workflow
    input = resource.Body("input")
    #: A Workflow definition using the Mistral v2 DSL
    definition = resource.Body("definition")
    #: A list of values associated with a workflow that users can use
    #: to group workflows by some criteria
    # TODO(briancurtin): type=list
    tags = resource.Body("tags")
    #: Can be either "private" or "public"
    scope = resource.Body("scope")
    #: The ID of the associated project
    project_id = resource.Body("project_id")
    #: The time at which the workflow was created
    created_at = resource.Body("created_at")
    #: The time at which the workflow was created
    updated_at = resource.Body("updated_at")

    def create(self, session, prepend_key=True, base_path=None):
        request = self._prepare_request(requires_id=False,
                                        prepend_key=prepend_key,
                                        base_path=base_path)

        headers = {
            "Content-Type": 'text/plain'
        }
        kwargs = {
            "data": self.definition,
        }

        scope = "?scope=%s" % self.scope
        uri = request.url + scope

        request.headers.update(headers)
        response = session.post(uri,
                                json=None,
                                headers=request.headers, **kwargs)

        self._translate_response(response, has_body=False)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.block_storage.v3 import capabilities
from openstack.tests.unit import base

CAPABILITIES = {
    "namespace": "OS::Storage::Capabilities::fake",
    "vendor_name": "OpenStack",
    "volume_backend_name": "lvmdriver-1",
    "pool_name": "pool",
    "driver_version": "2.0.0",
    "storage_protocol": "iSCSI",
    "display_name": "Capabilities of Cinder LVM driver",
    "description": "These are volume type options",
    "visibility": "public",
    "replication_targets": [],
    "properties": {
        "compression": {
            "title": "Compression",
            "description": "Enables compression.",
            "type": "boolean"
        },
        "qos": {
            "title": "QoS",
            "description": "Enables QoS.",
            "type": "boolean"
        },
        "replication": {
            "title": "Replication",
            "description": "Enables replication.",
            "type": "boolean"
        },
        "thin_provisioning": {
            "title": "Thin Provisioning",
            "description": "Sets thin provisioning.",
            "type": "boolean"
        }
    }
}


class TestCapabilites(base.TestCase):

    def test_basic(self):
        capabilities_resource = capabilities.Capabilities()
        self.assertEqual(None, capabilities_resource.resource_key)
        self.assertEqual(None, capabilities_resource.resources_key)
        self.assertEqual("/capabilities", capabilities_resource.base_path)
        self.assertTrue(capabilities_resource.allow_fetch)
        self.assertFalse(capabilities_resource.allow_create)
        self.assertFalse(capabilities_resource.allow_commit)
        self.assertFalse(capabilities_resource.allow_delete)
        self.assertFalse(capabilities_resource.allow_list)

    def test_make_capabilities(self):
        capabilities_resource = capabilities.Capabilities(**CAPABILITIES)
        self.assertEqual(
            CAPABILITIES["description"], capabilities_resource.description)
        self.assertEqual(
            CAPABILITIES["display_name"], capabilities_resource.display_name)
        self.assertEqual(
            CAPABILITIES["driver_version"],
            capabilities_resource.driver_version)
        self.assertEqual(
            CAPABILITIES["namespace"], capabilities_resource.namespace)
        self.assertEqual(
            CAPABILITIES["pool_name"], capabilities_resource.pool_name)
        self.assertEqual(
            CAPABILITIES["properties"], capabilities_resource.properties)
        self.assertEqual(
            CAPABILITIES["replication_targets"],
            capabilities_resource.replication_targets)
        self.assertEqual(
            CAPABILITIES["storage_protocol"],
            capabilities_resource.storage_protocol)
        self.assertEqual(
            CAPABILITIES["vendor_name"], capabilities_resource.vendor_name)
        self.assertEqual(
            CAPABILITIES["visibility"], capabilities_resource.visibility)
        self.assertEqual(
            CAPABILITIES["volume_backend_name"],
            capabilities_resource.volume_backend_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import metering_label_rule
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'direction': '1',
    'excluded': False,
    'id': IDENTIFIER,
    'metering_label_id': '4',
    'project_id': '5',
    'remote_ip_prefix': '6',
}


class TestMeteringLabelRule(base.TestCase):

    def test_basic(self):
        sot = metering_label_rule.MeteringLabelRule()
        self.assertEqual('metering_label_rule', sot.resource_key)
        self.assertEqual('metering_label_rules', sot.resources_key)
        self.assertEqual('/metering/metering-label-rules', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = metering_label_rule.MeteringLabelRule(**EXAMPLE)
        self.assertEqual(EXAMPLE['direction'], sot.direction)
        self.assertFalse(sot.is_excluded)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['metering_label_id'], sot.metering_label_id)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['remote_ip_prefix'], sot.remote_ip_prefix)

    def test_make_it_source_and_destination(self):
        custom_example = EXAMPLE.copy()
        custom_example["source_ip_prefix"] = "192.168.0.11/32"
        custom_example["destination_ip_prefix"] = "0.0.0.0/0"

        sot = metering_label_rule.MeteringLabelRule(**custom_example)
        self.assertEqual(custom_example['direction'], sot.direction)
        self.assertFalse(sot.is_excluded)
        self.assertEqual(custom_example['id'], sot.id)
        self.assertEqual(
            custom_example['metering_label_id'], sot.metering_label_id)
        self.assertEqual(custom_example['project_id'], sot.project_id)
        self.assertEqual(
            custom_example['remote_ip_prefix'], sot.remote_ip_prefix)

        self.assertEqual(
            custom_example['source_ip_prefix'], sot.source_ip_prefix)
        self.assertEqual(
            custom_example['destination_ip_prefix'], sot.destination_ip_prefix)



# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from io import StringIO
import logging
import os
import pprint
import sys

import fixtures
import munch
from oslotest import base
import testtools.content

_TRUE_VALUES = ('true', '1', 'yes')


class TestCase(base.BaseTestCase):

    """Test case base class for all tests."""

    # A way to adjust slow test classes
    TIMEOUT_SCALING_FACTOR = 1.0

    def setUp(self):
        """Run before each test method to initialize test environment."""
        # No openstacksdk unit tests should EVER run longer than a second.
        # Set this to 5 by default just to give us some fudge.
        # Do this before super setUp so that we intercept the default value
        # in oslotest. TODO(mordred) Make the default timeout configurable
        # in oslotest.
        test_timeout = int(os.environ.get('OS_TEST_TIMEOUT', '5'))
        try:
            test_timeout = int(test_timeout * self.TIMEOUT_SCALING_FACTOR)
            self.useFixture(
                fixtures.EnvironmentVariable(
                    'OS_TEST_TIMEOUT', str(test_timeout)))
        except ValueError:
            # Let oslotest do its thing
            pass

        super(TestCase, self).setUp()

        if os.environ.get('OS_LOG_CAPTURE') in _TRUE_VALUES:
            self._log_stream = StringIO()
            if os.environ.get('OS_ALWAYS_LOG') in _TRUE_VALUES:
                self.addCleanup(self.printLogs)
            else:
                self.addOnException(self.attachLogs)
        else:
            self._log_stream = sys.stdout

        handler = logging.StreamHandler(self._log_stream)
        formatter = logging.Formatter('%(asctime)s %(name)-32s %(message)s')
        handler.setFormatter(formatter)

        logger = logging.getLogger('openstack')
        logger.setLevel(logging.DEBUG)
        logger.addHandler(handler)

        # Enable HTTP level tracing
        # TODO(mordred) This is blowing out our memory we think
        logger = logging.getLogger('keystoneauth')
        logger.setLevel(logging.INFO)
        logger.addHandler(handler)
        logger.propagate = False

    def _fake_logs(self):
        # Override _fake_logs in oslotest until we can get our
        # attach-on-exception logic added
        pass

    def assertEqual(self, first, second, *args, **kwargs):
        '''Munch aware wrapper'''
        if isinstance(first, munch.Munch):
            first = first.toDict()
        if isinstance(second, munch.Munch):
            second = second.toDict()
        return super(TestCase, self).assertEqual(
            first, second, *args, **kwargs)

    def printLogs(self, *args):
        self._log_stream.seek(0)
        print(self._log_stream.read())

    def attachLogs(self, *args):
        def reader():
            self._log_stream.seek(0)
            while True:
                x = self._log_stream.read(4096)
                if not x:
                    break
                yield x.encode('utf8')
        content = testtools.content.content_from_reader(
            reader,
            testtools.content_type.UTF8_TEXT,
            False)
        self.addDetail('logging', content)

    def add_info_on_exception(self, name, text):
        def add_content(unused):
            self.addDetail(name, testtools.content.text_content(
                pprint.pformat(text)))
        self.addOnException(add_content)

    def assertSubdict(self, part, whole):
        missing_keys = []
        for key in part:
            # In the resource we have virtual access by not existing keys. To
            # verify those are there try access it.
            if not whole[key] and part[key]:
                missing_keys.append(key)
        if missing_keys:
            self.fail("Keys %s are in %s but not in %s" %
                      (missing_keys, part, whole))
        wrong_values = [(key, part[key], whole[key])
                        for key in part if part[key] != whole[key]]
        if wrong_values:
            self.fail("Mismatched values: %s" %
                      ", ".join("for %s got %s and %s" % tpl
                                for tpl in wrong_values))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack.tests.functional import base


class TestApplicationCredentials(base.BaseFunctionalTest):

    def setUp(self):
        super(TestApplicationCredentials, self).setUp()
        self.user_id = self.operator_cloud.current_user_id

    def _create_application_credentials(self):
        app_creds = self.conn.identity.create_application_credential(
            user=self.user_id, name='app_cred'
        )
        self.addCleanup(self.conn.identity.delete_application_credential,
                        self.user_id, app_creds['id'])
        return app_creds

    def test_create_application_credentials(self):
        app_creds = self._create_application_credentials()
        self.assertEqual(app_creds['user_id'], self.user_id)

    def test_get_application_credential(self):
        app_creds = self._create_application_credentials()
        app_cred = self.conn.identity.get_application_credential(
            user=self.user_id, application_credential=app_creds['id']
        )
        self.assertEqual(app_cred['id'], app_creds['id'])
        self.assertEqual(app_cred['user_id'], self.user_id)

    def test_application_credentials(self):
        self._create_application_credentials()
        app_creds = self.conn.identity.application_credentials(
            user=self.user_id
        )
        for app_cred in app_creds:
            self.assertEqual(app_cred['user_id'], self.user_id)

    def test_find_application_credential(self):
        app_creds = self._create_application_credentials()
        app_cred = self.conn.identity.find_application_credential(
            user=self.user_id, name_or_id=app_creds['id']
        )
        self.assertEqual(app_cred['id'], app_creds['id'])
        self.assertEqual(app_cred['user_id'], self.user_id)

    def test_delete_application_credential(self):
        app_creds = self._create_application_credentials()
        self.conn.identity.delete_application_credential(
            user=self.user_id, application_credential=app_creds['id']
        )
        self.assertRaises(exceptions.NotFoundException,
                          self.conn.identity.get_application_credential,
                          user=self.user_id,
                          application_credential=app_creds['id']
                          )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class DeviceProfile(resource.Resource):
    resource_key = 'device_profile'
    resources_key = 'device_profiles'
    base_path = '/device_profiles'
    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = False
    allow_delete = True
    allow_list = True

    #: The timestamp when this device_profile was created.
    created_at = resource.Body('created_at')
    #: The description of the device profile
    description = resource.Body('description')
    #: The groups of the device profile
    groups = resource.Body('groups')
    #: The name of the device profile
    name = resource.Body('name')
    #: The timestamp when this device_profile was updated.
    updated_at = resource.Body('updated_at')
    #: The uuid of the device profile
    uuid = resource.Body('uuid', alternate_id=True)

    # TODO(s_shogo): This implementation only treat [ DeviceProfile ], and
    # cannot treat multiple DeviceProfiles in list.
    def _prepare_request_body(self, patch, prepend_key):
        body = super(DeviceProfile, self)._prepare_request_body(
            patch, prepend_key)
        return [body]

    def create(self, session, base_path=None):
        # This overrides the default behavior of resource creation because
        # cyborg doesn't accept resource_key in its request.
        return super(DeviceProfile, self).create(
            session, prepend_key=False, base_path=base_path)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import pool_member
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'address': '1',
    'admin_state_up': True,
    'id': IDENTIFIER,
    'project_id': '4',
    'protocol_port': 5,
    'subnet_id': '6',
    'weight': 7,
    'name': '8',
    'pool_id': 'FAKE_POOL',
}


class TestPoolMember(base.TestCase):

    def test_basic(self):
        sot = pool_member.PoolMember()
        self.assertEqual('member', sot.resource_key)
        self.assertEqual('members', sot.resources_key)
        self.assertEqual('/lbaas/pools/%(pool_id)s/members', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = pool_member.PoolMember(**EXAMPLE)
        self.assertEqual(EXAMPLE['address'], sot.address)
        self.assertTrue(sot.is_admin_state_up)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['protocol_port'], sot.protocol_port)
        self.assertEqual(EXAMPLE['subnet_id'], sot.subnet_id)
        self.assertEqual(EXAMPLE['weight'], sot.weight)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['pool_id'], sot.pool_id)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_floating_ip_neutron
----------------------------------

Tests Floating IP resource methods for Neutron
"""

import copy
import datetime

import munch

from openstack.cloud import exc
from openstack.tests import fakes
from openstack.tests.unit import base


class TestFloatingIP(base.TestCase):
    mock_floating_ip_list_rep = {
        'floatingips': [
            {
                'router_id': 'd23abc8d-2991-4a55-ba98-2aaea84cc72f',
                'tenant_id': '4969c491a3c74ee4af974e6d800c62de',
                'floating_network_id': '376da547-b977-4cfe-9cba-275c80debf57',
                'fixed_ip_address': '10.0.0.4',
                'floating_ip_address': '172.24.4.229',
                'port_id': 'ce705c24-c1ef-408a-bda3-7bbd946164ac',
                'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda7',
                'status': 'ACTIVE'
            },
            {
                'router_id': None,
                'tenant_id': '4969c491a3c74ee4af974e6d800c62de',
                'floating_network_id': '376da547-b977-4cfe-9cba-275c80debf57',
                'fixed_ip_address': None,
                'floating_ip_address': '203.0.113.30',
                'port_id': None,
                'id': '61cea855-49cb-4846-997d-801b70c71bdd',
                'status': 'DOWN'
            }
        ]
    }

    mock_floating_ip_new_rep = {
        'floatingip': {
            'fixed_ip_address': '10.0.0.4',
            'floating_ip_address': '172.24.4.229',
            'floating_network_id': 'my-network-id',
            'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda8',
            'port_id': None,
            'router_id': None,
            'status': 'ACTIVE',
            'tenant_id': '4969c491a3c74ee4af974e6d800c62df'
        }
    }

    mock_floating_ip_port_rep = {
        'floatingip': {
            'fixed_ip_address': '10.0.0.4',
            'floating_ip_address': '172.24.4.229',
            'floating_network_id': 'my-network-id',
            'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda8',
            'port_id': 'ce705c24-c1ef-408a-bda3-7bbd946164ac',
            'router_id': None,
            'status': 'ACTIVE',
            'tenant_id': '4969c491a3c74ee4af974e6d800c62df'
        }
    }

    mock_get_network_rep = {
        'status': 'ACTIVE',
        'subnets': [
            '54d6f61d-db07-451c-9ab3-b9609b6b6f0b'
        ],
        'name': 'my-network',
        'provider:physical_network': None,
        'admin_state_up': True,
        'tenant_id': '4fd44f30292945e481c7b8a0c8908869',
        'provider:network_type': 'local',
        'router:external': True,
        'shared': True,
        'id': 'my-network-id',
        'provider:segmentation_id': None
    }

    mock_search_ports_rep = [
        {
            'status': 'ACTIVE',
            'binding:host_id': 'devstack',
            'name': 'first-port',
            'created_at': datetime.datetime.now().isoformat(),
            'allowed_address_pairs': [],
            'admin_state_up': True,
            'network_id': '70c1db1f-b701-45bd-96e0-a313ee3430b3',
            'tenant_id': '',
            'extra_dhcp_opts': [],
            'binding:vif_details': {
                'port_filter': True,
                'ovs_hybrid_plug': True
            },
            'binding:vif_type': 'ovs',
            'device_owner': 'compute:None',
            'mac_address': 'fa:16:3e:58:42:ed',
            'binding:profile': {},
            'binding:vnic_type': 'normal',
            'fixed_ips': [
                {
                    'subnet_id': '008ba151-0b8c-4a67-98b5-0d2b87666062',
                    'ip_address': u'172.24.4.2'
                }
            ],
            'id': 'ce705c24-c1ef-408a-bda3-7bbd946164ac',
            'security_groups': [],
            'device_id': 'server-id'
        }
    ]

    def assertAreInstances(self, elements, elem_type):
        for e in elements:
            self.assertIsInstance(e, elem_type)

    def setUp(self):
        super(TestFloatingIP, self).setUp()

        self.fake_server = fakes.make_fake_server(
            'server-id', '', 'ACTIVE',
            addresses={u'test_pnztt_net': [{
                u'OS-EXT-IPS:type': u'fixed',
                u'addr': '192.0.2.129',
                u'version': 4,
                u'OS-EXT-IPS-MAC:mac_addr':
                u'fa:16:3e:ae:7d:42'}]})
        self.floating_ip = self.cloud._normalize_floating_ips(
            self.mock_floating_ip_list_rep['floatingips'])[0]

    def test_float_no_status(self):
        floating_ips = [
            {
                'fixed_ip_address': '10.0.0.4',
                'floating_ip_address': '172.24.4.229',
                'floating_network_id': 'my-network-id',
                'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda8',
                'port_id': None,
                'router_id': None,
                'tenant_id': '4969c491a3c74ee4af974e6d800c62df'
            }
        ]
        normalized = self.cloud._normalize_floating_ips(floating_ips)
        self.assertEqual('UNKNOWN', normalized[0]['status'])

    def test_list_floating_ips(self):
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/floatingips',
                 json=self.mock_floating_ip_list_rep)])

        floating_ips = self.cloud.list_floating_ips()

        self.assertIsInstance(floating_ips, list)
        self.assertAreInstances(floating_ips, dict)
        self.assertEqual(2, len(floating_ips))

        self.assert_calls()

    def test_list_floating_ips_with_filters(self):

        self.register_uris([
            dict(method='GET',
                 uri=('https://network.example.com/v2.0/floatingips?'
                      'description=42'),
                 json={'floatingips': []})])

        self.cloud.list_floating_ips(filters={'description': 42})

        self.assert_calls()

    def test_search_floating_ips(self):
        self.register_uris([
            dict(method='GET',
                 uri=('https://network.example.com/v2.0/floatingips'),
                 json=self.mock_floating_ip_list_rep)])

        floating_ips = self.cloud.search_floating_ips(
            filters={'attached': False})

        self.assertIsInstance(floating_ips, list)
        self.assertAreInstances(floating_ips, dict)
        self.assertEqual(1, len(floating_ips))
        self.assert_calls()

    def test_get_floating_ip(self):
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/floatingips',
                 json=self.mock_floating_ip_list_rep)])

        floating_ip = self.cloud.get_floating_ip(
            id='2f245a7b-796b-4f26-9cf9-9e82d248fda7')

        self.assertIsInstance(floating_ip, dict)
        self.assertEqual('172.24.4.229', floating_ip['floating_ip_address'])
        self.assertEqual(
            self.mock_floating_ip_list_rep['floatingips'][0]['tenant_id'],
            floating_ip['project_id']
        )
        self.assertEqual(
            self.mock_floating_ip_list_rep['floatingips'][0]['tenant_id'],
            floating_ip['tenant_id']
        )
        self.assertIn('location', floating_ip)
        self.assert_calls()

    def test_get_floating_ip_not_found(self):
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/floatingips',
                 json=self.mock_floating_ip_list_rep)])

        floating_ip = self.cloud.get_floating_ip(id='non-existent')

        self.assertIsNone(floating_ip)
        self.assert_calls()

    def test_get_floating_ip_by_id(self):
        fid = self.mock_floating_ip_new_rep['floatingip']['id']
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/floatingips/'
                     '{id}'.format(id=fid),
                 json=self.mock_floating_ip_new_rep)])

        floating_ip = self.cloud.get_floating_ip_by_id(id=fid)

        self.assertIsInstance(floating_ip, dict)
        self.assertEqual('172.24.4.229', floating_ip['floating_ip_address'])
        self.assertEqual(
            self.mock_floating_ip_new_rep['floatingip']['tenant_id'],
            floating_ip['project_id']
        )
        self.assertEqual(
            self.mock_floating_ip_new_rep['floatingip']['tenant_id'],
            floating_ip['tenant_id']
        )
        self.assertIn('location', floating_ip)
        self.assert_calls()

    def test_create_floating_ip(self):
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks/my-network',
                 status_code=404),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks'
                     '?name=my-network',
                 json={'networks': [self.mock_get_network_rep]}),
            dict(method='POST',
                 uri='https://network.example.com/v2.0/floatingips',
                 json=self.mock_floating_ip_new_rep,
                 validate=dict(
                     json={'floatingip': {
                         'floating_network_id': 'my-network-id'}}))
        ])
        ip = self.cloud.create_floating_ip(network='my-network')

        self.assertEqual(
            self.mock_floating_ip_new_rep['floatingip']['floating_ip_address'],
            ip['floating_ip_address'])
        self.assert_calls()

    def test_create_floating_ip_port_bad_response(self):
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks/my-network',
                 json=self.mock_get_network_rep),
            dict(method='POST',
                 uri='https://network.example.com/v2.0/floatingips',
                 json=self.mock_floating_ip_new_rep,
                 validate=dict(
                     json={'floatingip': {
                         'floating_network_id': 'my-network-id',
                         'port_id': u'ce705c24-c1ef-408a-bda3-7bbd946164ab'}}))
        ])

        # Fails because we requested a port and the returned FIP has no port
        self.assertRaises(
            exc.OpenStackCloudException,
            self.cloud.create_floating_ip,
            network='my-network', port='ce705c24-c1ef-408a-bda3-7bbd946164ab')
        self.assert_calls()

    def test_create_floating_ip_port(self):
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks/my-network',
                 status_code=404),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks'
                     '?name=my-network',
                 json={'networks': [self.mock_get_network_rep]}),
            dict(method='POST',
                 uri='https://network.example.com/v2.0/floatingips',
                 json=self.mock_floating_ip_port_rep,
                 validate=dict(
                     json={'floatingip': {
                         'floating_network_id': 'my-network-id',
                         'port_id': u'ce705c24-c1ef-408a-bda3-7bbd946164ac'}}))
        ])

        ip = self.cloud.create_floating_ip(
            network='my-network', port='ce705c24-c1ef-408a-bda3-7bbd946164ac')

        self.assertEqual(
            self.mock_floating_ip_new_rep['floatingip']['floating_ip_address'],
            ip['floating_ip_address'])
        self.assert_calls()

    def test_neutron_available_floating_ips(self):
        """
        Test without specifying a network name.
        """
        fips_mock_uri = 'https://network.example.com/v2.0/floatingips'
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks',
                 json={'networks': [self.mock_get_network_rep]}),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/subnets',
                 json={'subnets': []}),
            dict(method='GET', uri=fips_mock_uri, json={'floatingips': []}),
            dict(method='POST', uri=fips_mock_uri,
                 json=self.mock_floating_ip_new_rep,
                 validate=dict(json={
                     'floatingip': {
                         'floating_network_id': self.mock_get_network_rep['id']
                     }}))
        ])

        # Test if first network is selected if no network is given
        self.cloud._neutron_available_floating_ips()
        self.assert_calls()

    def test_neutron_available_floating_ips_network(self):
        """
        Test with specifying a network name.
        """
        fips_mock_uri = 'https://network.example.com/v2.0/floatingips'
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks',
                 json={'networks': [self.mock_get_network_rep]}),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/subnets',
                 json={'subnets': []}),
            dict(method='GET', uri=fips_mock_uri, json={'floatingips': []}),
            dict(method='POST', uri=fips_mock_uri,
                 json=self.mock_floating_ip_new_rep,
                 validate=dict(json={
                     'floatingip': {
                         'floating_network_id': self.mock_get_network_rep['id']
                     }}))
        ])

        # Test if first network is selected if no network is given
        self.cloud._neutron_available_floating_ips(
            network=self.mock_get_network_rep['name']
        )
        self.assert_calls()

    def test_neutron_available_floating_ips_invalid_network(self):
        """
        Test with an invalid network name.
        """
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks',
                 json={'networks': [self.mock_get_network_rep]}),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/subnets',
                 json={'subnets': []})
        ])

        self.assertRaises(
            exc.OpenStackCloudException,
            self.cloud._neutron_available_floating_ips,
            network='INVALID')

        self.assert_calls()

    def test_auto_ip_pool_no_reuse(self):
        # payloads taken from citycloud
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks/ext-net',
                 status_code=404),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks?name=ext-net',
                 json={"networks": [{
                     "status": "ACTIVE",
                     "subnets": [
                         "df3e17fa-a4b2-47ae-9015-bc93eb076ba2",
                         "6b0c3dc9-b0b8-4d87-976a-7f2ebf13e7ec",
                         "fc541f48-fc7f-48c0-a063-18de6ee7bdd7"],
                     "availability_zone_hints": [],
                     "availability_zones": ["nova"],
                     "name": "ext-net",
                     "admin_state_up": True,
                     "tenant_id": "a564613210ee43708b8a7fc6274ebd63",
                     "tags": [],
                     "ipv6_address_scope": "9f03124f-89af-483a-b6fd-10f08079db4d",  # noqa
                     "mtu": 0,
                     "is_default": False,
                     "router:external": True,
                     "ipv4_address_scope": None,
                     "shared": False,
                     "id": "0232c17f-2096-49bc-b205-d3dcd9a30ebf",
                     "description": None
                 }]}),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/ports'
                     '?device_id=f80e3ad0-e13e-41d4-8e9c-be79bccdb8f7',
                 json={"ports": [{
                     "status": "ACTIVE",
                     "created_at": "2017-02-06T20:59:45",
                     "description": "",
                     "allowed_address_pairs": [],
                     "admin_state_up": True,
                     "network_id": "2c9adcb5-c123-4c5a-a2ba-1ad4c4e1481f",
                     "dns_name": None,
                     "extra_dhcp_opts": [],
                     "mac_address": "fa:16:3e:e8:7f:03",
                     "updated_at": "2017-02-06T20:59:49",
                     "name": "",
                     "device_owner": "compute:None",
                     "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                     "binding:vnic_type": "normal",
                     "fixed_ips": [{
                         "subnet_id": "f0ad1df5-53ee-473f-b86b-3604ea5591e9",
                         "ip_address": "10.4.0.16"}],
                     "id": "a767944e-057a-47d1-a669-824a21b8fb7b",
                     "security_groups": [
                         "9fb5ba44-5c46-4357-8e60-8b55526cab54"],
                     "device_id": "f80e3ad0-e13e-41d4-8e9c-be79bccdb8f7",
                 }]}),

            dict(method='POST',
                 uri='https://network.example.com/v2.0/floatingips',
                 json={"floatingip": {
                     "router_id": "9de9c787-8f89-4a53-8468-a5533d6d7fd1",
                     "status": "DOWN",
                     "description": "",
                     "dns_domain": "",
                     "floating_network_id": "0232c17f-2096-49bc-b205-d3dcd9a30ebf",  # noqa
                     "fixed_ip_address": "10.4.0.16",
                     "floating_ip_address": "89.40.216.153",
                     "port_id": "a767944e-057a-47d1-a669-824a21b8fb7b",
                     "id": "e69179dc-a904-4c9a-a4c9-891e2ecb984c",
                     "dns_name": "",
                     "tenant_id": "65222a4d09ea4c68934fa1028c77f394"
                 }},
                 validate=dict(json={"floatingip": {
                     "floating_network_id": "0232c17f-2096-49bc-b205-d3dcd9a30ebf",  # noqa
                     "fixed_ip_address": "10.4.0.16",
                     "port_id": "a767944e-057a-47d1-a669-824a21b8fb7b",
                 }})),
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri='{endpoint}/servers/detail'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json={"servers": [{
                     "status": "ACTIVE",
                     "updated": "2017-02-06T20:59:49Z",
                     "addresses": {
                         "private": [{
                             "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:e8:7f:03",
                             "version": 4,
                             "addr": "10.4.0.16",
                             "OS-EXT-IPS:type": "fixed"
                         }, {
                             "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:e8:7f:03",
                             "version": 4,
                             "addr": "89.40.216.153",
                             "OS-EXT-IPS:type": "floating"
                         }]},
                     "key_name": None,
                     "image": {"id": "95e4c449-8abf-486e-97d9-dc3f82417d2d"},
                     "OS-EXT-STS:task_state": None,
                     "OS-EXT-STS:vm_state": "active",
                     "OS-SRV-USG:launched_at": "2017-02-06T20:59:48.000000",
                     "flavor": {"id": "2186bd79-a05e-4953-9dde-ddefb63c88d4"},
                     "id": "f80e3ad0-e13e-41d4-8e9c-be79bccdb8f7",
                     "security_groups": [{"name": "default"}],
                     "OS-SRV-USG:terminated_at": None,
                     "OS-EXT-AZ:availability_zone": "nova",
                     "user_id": "c17534835f8f42bf98fc367e0bf35e09",
                     "name": "testmt",
                     "created": "2017-02-06T20:59:44Z",
                     "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                     "OS-DCF:diskConfig": "MANUAL",
                     "os-extended-volumes:volumes_attached": [],
                     "accessIPv4": "",
                     "accessIPv6": "",
                     "progress": 0,
                     "OS-EXT-STS:power_state": 1,
                     "config_drive": "",
                     "metadata": {}
                 }]}),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks',
                 json={"networks": [{
                     "status": "ACTIVE",
                     "subnets": [
                         "df3e17fa-a4b2-47ae-9015-bc93eb076ba2",
                         "6b0c3dc9-b0b8-4d87-976a-7f2ebf13e7ec",
                         "fc541f48-fc7f-48c0-a063-18de6ee7bdd7"],
                     "availability_zone_hints": [],
                     "availability_zones": ["nova"],
                     "name": "ext-net",
                     "admin_state_up": True,
                     "tenant_id": "a564613210ee43708b8a7fc6274ebd63",
                     "tags": [],
                     "ipv6_address_scope": "9f03124f-89af-483a-b6fd-10f08079db4d",  # noqa
                     "mtu": 0,
                     "is_default": False,
                     "router:external": True,
                     "ipv4_address_scope": None,
                     "shared": False,
                     "id": "0232c17f-2096-49bc-b205-d3dcd9a30ebf",
                     "description": None
                 }, {
                     "status": "ACTIVE",
                     "subnets": ["f0ad1df5-53ee-473f-b86b-3604ea5591e9"],
                     "availability_zone_hints": [],
                     "availability_zones": ["nova"],
                     "name": "private",
                     "admin_state_up": True,
                     "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                     "created_at": "2016-10-22T13:46:26",
                     "tags": [],
                     "updated_at": "2016-10-22T13:46:26",
                     "ipv6_address_scope": None,
                     "router:external": False,
                     "ipv4_address_scope": None,
                     "shared": False,
                     "mtu": 1450,
                     "id": "2c9adcb5-c123-4c5a-a2ba-1ad4c4e1481f",
                     "description": ""
                 }]}),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/subnets',
                 json={"subnets": [{
                     "description": "",
                     "enable_dhcp": True,
                     "network_id": "2c9adcb5-c123-4c5a-a2ba-1ad4c4e1481f",
                     "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                     "created_at": "2016-10-22T13:46:26",
                     "dns_nameservers": [
                         "89.36.90.101",
                         "89.36.90.102"],
                     "updated_at": "2016-10-22T13:46:26",
                     "gateway_ip": "10.4.0.1",
                     "ipv6_ra_mode": None,
                     "allocation_pools": [{
                         "start": "10.4.0.2",
                         "end": "10.4.0.200"}],
                     "host_routes": [],
                     "ip_version": 4,
                     "ipv6_address_mode": None,
                     "cidr": "10.4.0.0/24",
                     "id": "f0ad1df5-53ee-473f-b86b-3604ea5591e9",
                     "subnetpool_id": None,
                     "name": "private-subnet-ipv4",
                 }]})])

        self.cloud.add_ips_to_server(
            munch.Munch(
                id='f80e3ad0-e13e-41d4-8e9c-be79bccdb8f7',
                addresses={
                    "private": [{
                        "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:e8:7f:03",
                        "version": 4,
                        "addr": "10.4.0.16",
                        "OS-EXT-IPS:type": "fixed"
                    }]}),
            ip_pool='ext-net', reuse=False)

        self.assert_calls()

    def test_available_floating_ip_new(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks']),
                 json={'networks': [self.mock_get_network_rep]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets']),
                 json={'subnets': []}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': []}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 validate=dict(
                     json={'floatingip': {
                         'floating_network_id': 'my-network-id'}}),
                 json=self.mock_floating_ip_new_rep)
        ])

        ip = self.cloud.available_floating_ip(network='my-network')

        self.assertEqual(
            self.mock_floating_ip_new_rep['floatingip']['floating_ip_address'],
            ip['floating_ip_address'])
        self.assert_calls()

    def test_delete_floating_ip_existing(self):
        fip_id = '2f245a7b-796b-4f26-9cf9-9e82d248fda7'
        fake_fip = {
            'id': fip_id,
            'floating_ip_address': '172.99.106.167',
            'status': 'ACTIVE',
        }
        self.register_uris([
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(fip_id)]),
                 json={}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [fake_fip]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(fip_id)]),
                 json={}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [fake_fip]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(fip_id)]),
                 json={}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': []}),
        ])

        self.assertTrue(
            self.cloud.delete_floating_ip(floating_ip_id=fip_id, retry=2))
        self.assert_calls()

    def test_delete_floating_ip_existing_down(self):
        fip_id = '2f245a7b-796b-4f26-9cf9-9e82d248fda7'
        fake_fip = {
            'id': fip_id,
            'floating_ip_address': '172.99.106.167',
            'status': 'ACTIVE',
        }
        down_fip = {
            'id': fip_id,
            'floating_ip_address': '172.99.106.167',
            'status': 'DOWN',
        }
        self.register_uris([
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(fip_id)]),
                 json={}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [fake_fip]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(fip_id)]),
                 json={}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [down_fip]}),
        ])

        self.assertTrue(
            self.cloud.delete_floating_ip(floating_ip_id=fip_id, retry=2))
        self.assert_calls()

    def test_delete_floating_ip_existing_no_delete(self):
        fip_id = '2f245a7b-796b-4f26-9cf9-9e82d248fda7'
        fake_fip = {
            'id': fip_id,
            'floating_ip_address': '172.99.106.167',
            'status': 'ACTIVE',
        }
        self.register_uris([
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(fip_id)]),
                 json={}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [fake_fip]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(fip_id)]),
                 json={}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [fake_fip]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(fip_id)]),
                 json={}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [fake_fip]}),
        ])
        self.assertRaises(
            exc.OpenStackCloudException,
            self.cloud.delete_floating_ip,
            floating_ip_id=fip_id, retry=2)
        self.assert_calls()

    def test_delete_floating_ip_not_found(self):
        self.register_uris([
            dict(method='DELETE',
                 uri=('https://network.example.com/v2.0/floatingips/'
                      'a-wild-id-appears'),
                 status_code=404)])

        ret = self.cloud.delete_floating_ip(
            floating_ip_id='a-wild-id-appears')

        self.assertFalse(ret)
        self.assert_calls()

    def test_attach_ip_to_server(self):
        fip = self.mock_floating_ip_list_rep['floatingips'][0].copy()
        fip.update({'status': 'DOWN', 'port_id': None, 'router_id': None})
        device_id = self.fake_server['id']

        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'ports'],
                     qs_elements=["device_id={0}".format(device_id)]),
                 json={'ports': self.mock_search_ports_rep}),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(
                         fip['id'])]),
                 json={'floatingip':
                       self.mock_floating_ip_list_rep['floatingips'][0]},
                 validate=dict(
                     json={'floatingip': {
                         'port_id': self.mock_search_ports_rep[0]['id'],
                         'fixed_ip_address': self.mock_search_ports_rep[0][
                             'fixed_ips'][0]['ip_address']}})),
        ])

        self.cloud._attach_ip_to_server(
            server=self.fake_server,
            floating_ip=self.cloud._normalize_floating_ip(fip))
        self.assert_calls()

    def test_detach_ip_from_server(self):
        fip = self.mock_floating_ip_new_rep['floatingip']
        attached_fip = copy.copy(fip)
        attached_fip['port_id'] = 'server-port-id'
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [attached_fip]}),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(
                         fip['id'])]),
                 json={'floatingip': fip},
                 validate=dict(
                     json={'floatingip': {'port_id': None}}))
        ])
        self.cloud.detach_ip_from_server(
            server_id='server-id',
            floating_ip_id=fip['id'])
        self.assert_calls()

    def test_add_ip_from_pool(self):
        network = self.mock_get_network_rep
        fip = self.mock_floating_ip_new_rep['floatingip']
        fixed_ip = self.mock_search_ports_rep[0]['fixed_ips'][0]['ip_address']
        port_id = self.mock_search_ports_rep[0]['id']
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks']),
                 json={'networks': [network]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets']),
                 json={'subnets': []}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [fip]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingip': fip},
                 validate=dict(
                     json={'floatingip': {
                         'floating_network_id': network['id']}})),
            dict(method="GET",
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'ports'],
                     qs_elements=[
                        "device_id={0}".format(self.fake_server['id'])]),
                 json={'ports': self.mock_search_ports_rep}),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(
                         fip['id'])]),
                 json={'floatingip': fip},
                 validate=dict(
                     json={'floatingip': {
                         'fixed_ip_address': fixed_ip,
                         'port_id': port_id}})),
        ])

        server = self.cloud._add_ip_from_pool(
            server=self.fake_server,
            network=network['id'],
            fixed_address=fixed_ip)

        self.assertEqual(server, self.fake_server)
        self.assert_calls()

    def test_cleanup_floating_ips(self):
        floating_ips = [{
            "id": "this-is-a-floating-ip-id",
            "fixed_ip_address": None,
            "internal_network": None,
            "floating_ip_address": "203.0.113.29",
            "network": "this-is-a-net-or-pool-id",
            "port_id": None,
            "status": "ACTIVE"
        }, {
            "id": "this-is-a-second-floating-ip-id",
            "fixed_ip_address": None,
            "internal_network": None,
            "floating_ip_address": "203.0.113.30",
            "network": "this-is-a-net-or-pool-id",
            "port_id": None,
            "status": "ACTIVE"
        }, {
            "id": "this-is-an-attached-floating-ip-id",
            "fixed_ip_address": None,
            "internal_network": None,
            "floating_ip_address": "203.0.113.29",
            "network": "this-is-a-net-or-pool-id",
            "attached": True,
            "port_id": "this-is-id-of-port-with-fip",
            "status": "ACTIVE"
        }]
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': floating_ips}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(
                         floating_ips[0]['id'])]),
                 json={}),
            # First IP has been deleted now, return just the second
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': floating_ips[1:]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips/{0}'.format(
                         floating_ips[1]['id'])]),
                 json={}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingips': [floating_ips[2]]}),
        ])
        cleaned_up = self.cloud.delete_unattached_floating_ips()
        self.assertEqual(cleaned_up, 2)
        self.assert_calls()

    def test_create_floating_ip_no_port(self):
        server_port = {
            "id": "port-id",
            "device_id": "some-server",
            'created_at': datetime.datetime.now().isoformat(),
            'fixed_ips': [
                {
                    'subnet_id': 'subnet-id',
                    'ip_address': '172.24.4.2'
                }
            ],
        }
        floating_ip = {
            "id": "floating-ip-id",
            "port_id": None
        }
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks']),
                 json={'networks': [self.mock_get_network_rep]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets']),
                 json={'subnets': []}),
            dict(method="GET",
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'ports'],
                     qs_elements=['device_id=some-server']),
                 json={'ports': [server_port]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 json={'floatingip': floating_ip})
        ])

        self.assertRaises(
            exc.OpenStackCloudException,
            self.cloud._neutron_create_floating_ip,
            server=dict(id='some-server'))
        self.assert_calls()

    def test_find_nat_source_inferred(self):
        # payloads contrived but based on ones from citycloud
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks',
                 json={"networks": [{
                     "status": "ACTIVE",
                     "subnets": [
                         "df3e17fa-a4b2-47ae-9015-bc93eb076ba2",
                         "6b0c3dc9-b0b8-4d87-976a-7f2ebf13e7ec",
                         "fc541f48-fc7f-48c0-a063-18de6ee7bdd7"],
                     "availability_zone_hints": [],
                     "availability_zones": ["nova"],
                     "name": "ext-net",
                     "admin_state_up": True,
                     "tenant_id": "a564613210ee43708b8a7fc6274ebd63",
                     "tags": [],
                     "ipv6_address_scope": "9f03124f-89af-483a-b6fd-10f08079db4d",  # noqa
                     "mtu": 0,
                     "is_default": False,
                     "router:external": True,
                     "ipv4_address_scope": None,
                     "shared": False,
                     "id": "0232c17f-2096-49bc-b205-d3dcd9a30ebf",
                     "description": None
                 }, {
                     "status": "ACTIVE",
                     "subnets": [
                         "df3e17fa-a4b2-47ae-9015-bc93eb076ba2",
                         "6b0c3dc9-b0b8-4d87-976a-7f2ebf13e7ec",
                         "fc541f48-fc7f-48c0-a063-18de6ee7bdd7"],
                     "availability_zone_hints": [],
                     "availability_zones": ["nova"],
                     "name": "my-network",
                     "admin_state_up": True,
                     "tenant_id": "a564613210ee43708b8a7fc6274ebd63",
                     "tags": [],
                     "ipv6_address_scope": "9f03124f-89af-483a-b6fd-10f08079db4d",  # noqa
                     "mtu": 0,
                     "is_default": False,
                     "router:external": True,
                     "ipv4_address_scope": None,
                     "shared": False,
                     "id": "0232c17f-2096-49bc-b205-d3dcd9a30ebg",
                     "description": None
                 }, {
                     "status": "ACTIVE",
                     "subnets": ["f0ad1df5-53ee-473f-b86b-3604ea5591e9"],
                     "availability_zone_hints": [],
                     "availability_zones": ["nova"],
                     "name": "private",
                     "admin_state_up": True,
                     "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                     "created_at": "2016-10-22T13:46:26",
                     "tags": [],
                     "updated_at": "2016-10-22T13:46:26",
                     "ipv6_address_scope": None,
                     "router:external": False,
                     "ipv4_address_scope": None,
                     "shared": False,
                     "mtu": 1450,
                     "id": "2c9adcb5-c123-4c5a-a2ba-1ad4c4e1481f",
                     "description": ""
                 }]}),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/subnets',
                 json={"subnets": [{
                     "description": "",
                     "enable_dhcp": True,
                     "network_id": "2c9adcb5-c123-4c5a-a2ba-1ad4c4e1481f",
                     "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                     "created_at": "2016-10-22T13:46:26",
                     "dns_nameservers": [
                         "89.36.90.101",
                         "89.36.90.102"],
                     "updated_at": "2016-10-22T13:46:26",
                     "gateway_ip": "10.4.0.1",
                     "ipv6_ra_mode": None,
                     "allocation_pools": [{
                         "start": "10.4.0.2",
                         "end": "10.4.0.200"}],
                     "host_routes": [],
                     "ip_version": 4,
                     "ipv6_address_mode": None,
                     "cidr": "10.4.0.0/24",
                     "id": "f0ad1df5-53ee-473f-b86b-3604ea5591e9",
                     "subnetpool_id": None,
                     "name": "private-subnet-ipv4",
                 }]})
        ])

        self.assertEqual(
            'ext-net', self.cloud.get_nat_source()['name'])

        self.assert_calls()

    def test_find_nat_source_config(self):
        self.cloud._nat_source = 'my-network'

        # payloads contrived but based on ones from citycloud
        self.register_uris([
            dict(method='GET',
                 uri='https://network.example.com/v2.0/networks',
                 json={"networks": [{
                     "status": "ACTIVE",
                     "subnets": [
                         "df3e17fa-a4b2-47ae-9015-bc93eb076ba2",
                         "6b0c3dc9-b0b8-4d87-976a-7f2ebf13e7ec",
                         "fc541f48-fc7f-48c0-a063-18de6ee7bdd7"],
                     "availability_zone_hints": [],
                     "availability_zones": ["nova"],
                     "name": "ext-net",
                     "admin_state_up": True,
                     "tenant_id": "a564613210ee43708b8a7fc6274ebd63",
                     "tags": [],
                     "ipv6_address_scope": "9f03124f-89af-483a-b6fd-10f08079db4d",  # noqa
                     "mtu": 0,
                     "is_default": False,
                     "router:external": True,
                     "ipv4_address_scope": None,
                     "shared": False,
                     "id": "0232c17f-2096-49bc-b205-d3dcd9a30ebf",
                     "description": None
                 }, {
                     "status": "ACTIVE",
                     "subnets": [
                         "df3e17fa-a4b2-47ae-9015-bc93eb076ba2",
                         "6b0c3dc9-b0b8-4d87-976a-7f2ebf13e7ec",
                         "fc541f48-fc7f-48c0-a063-18de6ee7bdd7"],
                     "availability_zone_hints": [],
                     "availability_zones": ["nova"],
                     "name": "my-network",
                     "admin_state_up": True,
                     "tenant_id": "a564613210ee43708b8a7fc6274ebd63",
                     "tags": [],
                     "ipv6_address_scope": "9f03124f-89af-483a-b6fd-10f08079db4d",  # noqa
                     "mtu": 0,
                     "is_default": False,
                     "router:external": True,
                     "ipv4_address_scope": None,
                     "shared": False,
                     "id": "0232c17f-2096-49bc-b205-d3dcd9a30ebg",
                     "description": None
                 }, {
                     "status": "ACTIVE",
                     "subnets": ["f0ad1df5-53ee-473f-b86b-3604ea5591e9"],
                     "availability_zone_hints": [],
                     "availability_zones": ["nova"],
                     "name": "private",
                     "admin_state_up": True,
                     "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                     "created_at": "2016-10-22T13:46:26",
                     "tags": [],
                     "updated_at": "2016-10-22T13:46:26",
                     "ipv6_address_scope": None,
                     "router:external": False,
                     "ipv4_address_scope": None,
                     "shared": False,
                     "mtu": 1450,
                     "id": "2c9adcb5-c123-4c5a-a2ba-1ad4c4e1481f",
                     "description": ""
                 }]}),
            dict(method='GET',
                 uri='https://network.example.com/v2.0/subnets',
                 json={"subnets": [{
                     "description": "",
                     "enable_dhcp": True,
                     "network_id": "2c9adcb5-c123-4c5a-a2ba-1ad4c4e1481f",
                     "tenant_id": "65222a4d09ea4c68934fa1028c77f394",
                     "created_at": "2016-10-22T13:46:26",
                     "dns_nameservers": [
                         "89.36.90.101",
                         "89.36.90.102"],
                     "updated_at": "2016-10-22T13:46:26",
                     "gateway_ip": "10.4.0.1",
                     "ipv6_ra_mode": None,
                     "allocation_pools": [{
                         "start": "10.4.0.2",
                         "end": "10.4.0.200"}],
                     "host_routes": [],
                     "ip_version": 4,
                     "ipv6_address_mode": None,
                     "cidr": "10.4.0.0/24",
                     "id": "f0ad1df5-53ee-473f-b86b-3604ea5591e9",
                     "subnetpool_id": None,
                     "name": "private-subnet-ipv4",
                 }]})
        ])

        self.assertEqual(
            'my-network', self.cloud.get_nat_source()['name'])

        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import extension
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'alias': '1',
    'description': '2',
    'links': '3',
    'name': '4',
    'updated': '2016-03-09T12:14:57.233772',
}


class TestExtension(base.TestCase):

    def test_basic(self):
        sot = extension.Extension()
        self.assertEqual('extension', sot.resource_key)
        self.assertEqual('extensions', sot.resources_key)
        self.assertEqual('/extensions', sot.base_path)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = extension.Extension(**EXAMPLE)
        self.assertEqual(EXAMPLE['alias'], sot.id)
        self.assertEqual(EXAMPLE['alias'], sot.alias)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['links'], sot.links)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['updated'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_delete_server
----------------------------------

Tests for the `delete_server` command.
"""
import uuid

from openstack.cloud import exc as shade_exc
from openstack.tests import fakes
from openstack.tests.unit import base


class TestDeleteServer(base.TestCase):

    def test_delete_server(self):
        """
        Test that server delete is called when wait=False
        """
        server = fakes.make_fake_server('1234', 'daffy', 'ACTIVE')
        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'daffy']),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers'],
                     qs_elements=['name=daffy']),
                 json={'servers': [server]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234'])),
        ])
        self.assertTrue(self.cloud.delete_server('daffy', wait=False))

        self.assert_calls()

    def test_delete_server_already_gone(self):
        """
        Test that we return immediately when server is already gone
        """
        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'tweety']),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers'],
                     qs_elements=['name=tweety']),
                 json={'servers': []}),
        ])
        self.assertFalse(self.cloud.delete_server('tweety', wait=False))

        self.assert_calls()

    def test_delete_server_already_gone_wait(self):
        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'speedy']),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers'],
                     qs_elements=['name=speedy']),
                 json={'servers': []}),
        ])
        self.assertFalse(self.cloud.delete_server('speedy', wait=True))
        self.assert_calls()

    def test_delete_server_wait_for_deleted(self):
        """
        Test that delete_server waits for the server to be gone
        """
        server = fakes.make_fake_server('9999', 'wily', 'ACTIVE')
        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'wily']),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers'],
                     qs_elements=['name=wily']),
                 json={'servers': [server]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '9999'])),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '9999']),
                 json={'server': server}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '9999']),
                 status_code=404),
        ])
        self.assertTrue(self.cloud.delete_server('wily', wait=True))

        self.assert_calls()

    def test_delete_server_fails(self):
        """
        Test that delete_server raises non-404 exceptions
        """
        server = fakes.make_fake_server('1212', 'speedy', 'ACTIVE')
        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'speedy']),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers'],
                     qs_elements=['name=speedy']),
                 json={'servers': [server]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1212']),
                 status_code=400),
        ])

        self.assertRaises(
            shade_exc.OpenStackCloudException,
            self.cloud.delete_server, 'speedy',
            wait=False)

        self.assert_calls()

    def test_delete_server_no_cinder(self):
        """
        Test that deleting server works when cinder is not available
        """
        orig_has_service = self.cloud.has_service

        def fake_has_service(service_type):
            if service_type == 'volume':
                return False
            return orig_has_service(service_type)
        self.cloud.has_service = fake_has_service

        server = fakes.make_fake_server('1234', 'porky', 'ACTIVE')
        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'porky']),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers'],
                     qs_elements=['name=porky']),
                 json={'servers': [server]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234'])),
        ])
        self.assertTrue(self.cloud.delete_server('porky', wait=False))

        self.assert_calls()

    def test_delete_server_delete_ips(self):
        """
        Test that deleting server and fips works
        """
        server = fakes.make_fake_server('1234', 'porky', 'ACTIVE')
        fip_id = uuid.uuid4().hex

        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'porky']),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers'],
                     qs_elements=['name=porky']),
                 json={'servers': [server]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips'],
                     qs_elements=['floating_ip_address=172.24.5.5']),
                 complete_qs=True,
                 json={'floatingips': [{
                     'router_id': 'd23abc8d-2991-4a55-ba98-2aaea84cc72f',
                     'tenant_id': '4969c491a3c74ee4af974e6d800c62de',
                     'floating_network_id': '376da547-b977-4cfe-9cba7',
                     'fixed_ip_address': '10.0.0.4',
                     'floating_ip_address': '172.24.5.5',
                     'port_id': 'ce705c24-c1ef-408a-bda3-7bbd946164ac',
                     'id': fip_id,
                     'status': 'ACTIVE'}]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'floatingips', fip_id])),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips']),
                 complete_qs=True,
                 json={'floatingips': []}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234'])),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234']),
                 status_code=404),
        ])
        self.assertTrue(self.cloud.delete_server(
            'porky', wait=True, delete_ips=True))

        self.assert_calls()

    def test_delete_server_delete_ips_bad_neutron(self):
        """
        Test that deleting server with a borked neutron doesn't bork
        """
        server = fakes.make_fake_server('1234', 'porky', 'ACTIVE')

        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'porky']),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers'],
                     qs_elements=['name=porky']),
                 json={'servers': [server]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'floatingips'],
                     qs_elements=['floating_ip_address=172.24.5.5']),
                 complete_qs=True,
                 status_code=404),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234'])),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234']),
                 status_code=404),
        ])
        self.assertTrue(self.cloud.delete_server(
            'porky', wait=True, delete_ips=True))

        self.assert_calls()

    def test_delete_server_delete_fips_nova(self):
        """
        Test that deleting server with a borked neutron doesn't bork
        """
        self.cloud._floating_ip_source = 'nova'
        server = fakes.make_fake_server('1234', 'porky', 'ACTIVE')

        self.register_uris([
            self.get_nova_discovery_mock_dict(),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'porky']),
                 status_code=404),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers'],
                     qs_elements=['name=porky']),
                 json={'servers': [server]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-floating-ips']),
                 json={'floating_ips': [
                     {
                         'fixed_ip': None,
                         'id': 1,
                         'instance_id': None,
                         'ip': '172.24.5.5',
                         'pool': 'nova'
                     }]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-floating-ips', '1'])),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-floating-ips']),
                 json={'floating_ips': []}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234'])),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234']),
                 status_code=404),
        ])
        self.assertTrue(self.cloud.delete_server(
            'porky', wait=True, delete_ips=True))

        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.network.v2 import security_group
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
RULES = [
    {
        "remote_group_id": None,
        "direction": "egress",
        "remote_ip_prefix": None,
        "protocol": None,
        "ethertype":
        "IPv6",
        "project_id": "4",
        "port_range_max": None,
        "port_range_min": None,
        "id": "5",
        "security_group_id": IDENTIFIER,
        "created_at": "2016-10-04T12:14:57.233772",
        "updated_at": "2016-10-12T12:15:34.233222",
        "revision_number": 6,
    },
    {
        "remote_group_id": "9",
        "direction": "ingress",
        "remote_ip_prefix": None,
        "protocol": None,
        "ethertype": "IPv6",
        "project_id": "4",
        "port_range_max": None,
        "port_range_min": None,
        "id": "6",
        "security_group_id": IDENTIFIER,
        "created_at": "2016-10-04T12:14:57.233772",
        "updated_at": "2016-10-12T12:15:34.233222",
        "revision_number": 7,
    },
]

EXAMPLE = {
    'created_at': '2016-10-04T12:14:57.233772',
    'description': '1',
    'id': IDENTIFIER,
    'name': '2',
    'stateful': True,
    'revision_number': 3,
    'security_group_rules': RULES,
    'project_id': '4',
    'project_id': '4',
    'updated_at': '2016-10-14T12:16:57.233772',
    'tags': ['5']
}


class TestSecurityGroup(base.TestCase):

    def test_basic(self):
        sot = security_group.SecurityGroup()
        self.assertEqual('security_group', sot.resource_key)
        self.assertEqual('security_groups', sot.resources_key)
        self.assertEqual('/security-groups', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual({'any_tags': 'tags-any',
                              'description': 'description',
                              'fields': 'fields',
                              'id': 'id',
                              'limit': 'limit',
                              'marker': 'marker',
                              'name': 'name',
                              'not_any_tags': 'not-tags-any',
                              'not_tags': 'not-tags',
                              'tenant_id': 'tenant_id',
                              'revision_number': 'revision_number',
                              'sort_dir': 'sort_dir',
                              'sort_key': 'sort_key',
                              'tags': 'tags',
                              'project_id': 'project_id',
                              'stateful': 'stateful',
                              },
                             sot._query_mapping._mapping)

    def test_make_it(self):
        sot = security_group.SecurityGroup(**EXAMPLE)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['revision_number'], sot.revision_number)
        self.assertEqual(EXAMPLE['security_group_rules'],
                         sot.security_group_rules)
        self.assertEqual(dict, type(sot.security_group_rules[0]))
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE['tags'], sot.tags)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.block_storage.v2 import type as _type
from openstack.tests.functional.block_storage.v2 import base


class TestType(base.BaseBlockStorageTest):

    def setUp(self):
        super(TestType, self).setUp()

        self.TYPE_NAME = self.getUniqueString()
        self.TYPE_ID = None

        sot = self.operator_cloud.block_storage.create_type(
            name=self.TYPE_NAME)
        assert isinstance(sot, _type.Type)
        self.assertEqual(self.TYPE_NAME, sot.name)
        self.TYPE_ID = sot.id

    def tearDown(self):
        sot = self.operator_cloud.block_storage.delete_type(
            self.TYPE_ID, ignore_missing=False)
        self.assertIsNone(sot)
        super(TestType, self).tearDown()

    def test_get(self):
        sot = self.operator_cloud.block_storage.get_type(self.TYPE_ID)
        self.assertEqual(self.TYPE_NAME, sot.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.dns.v2 import zone_transfer
from openstack.tests.unit import base


IDENTIFIER = '074e805e-fe87-4cbb-b10b-21a06e215d41'
EXAMPLE_REQUEST = {
    'created_at': '2014-07-17T20:34:40.882579',
    'description': 'some description',
    'id': IDENTIFIER,
    'key': '9Z2R50Y0',
    'project_id': '1',
    'status': 'ACTIVE',
    'target_project_id': '123456',
    'updated_at': None,
    'zone_id': '6b78734a-aef1-45cd-9708-8eb3c2d26ff8',
    'zone_name': 'qa.dev.example.com.',
}
EXAMPLE_ACCEPT = {
    'status': 'COMPLETE',
    'zone_id': 'b4542f5a-f1ea-4ec1-b850-52db9dc3f465',
    'created_at': '2016-06-22 06:13:55',
    'updated_at': 'null',
    'key': 'FUGXMZ5N',
    'project_id': '2e43de7ce3504a8fb90a45382532c37e',
    'id': IDENTIFIER,
    'zone_transfer_request_id': '794fdf58-6e1d-41da-8b2d-16b6d10c8827'
}


class TestZoneTransferRequest(base.TestCase):

    def test_basic(self):
        sot = zone_transfer.ZoneTransferRequest()
        # self.assertEqual('', sot.resource_key)
        self.assertEqual('transfer_requests', sot.resources_key)
        self.assertEqual('/zones/tasks/transfer_requests', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

        self.assertDictEqual({'limit': 'limit',
                              'marker': 'marker',
                              'status': 'status'},
                             sot._query_mapping._mapping)

    def test_make_it(self):
        sot = zone_transfer.ZoneTransferRequest(**EXAMPLE_REQUEST)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE_REQUEST['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE_REQUEST['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE_REQUEST['description'], sot.description)
        self.assertEqual(EXAMPLE_REQUEST['key'], sot.key)
        self.assertEqual(EXAMPLE_REQUEST['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE_REQUEST['status'], sot.status)
        self.assertEqual(EXAMPLE_REQUEST['target_project_id'],
                         sot.target_project_id)
        self.assertEqual(EXAMPLE_REQUEST['zone_id'], sot.zone_id)
        self.assertEqual(EXAMPLE_REQUEST['zone_name'], sot.zone_name)


class TestZoneTransferAccept(base.TestCase):

    def test_basic(self):
        sot = zone_transfer.ZoneTransferAccept()
        # self.assertEqual('', sot.resource_key)
        self.assertEqual('transfer_accepts', sot.resources_key)
        self.assertEqual('/zones/tasks/transfer_accepts', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

        self.assertDictEqual({'limit': 'limit',
                              'marker': 'marker',
                              'status': 'status'},
                             sot._query_mapping._mapping)

    def test_make_it(self):
        sot = zone_transfer.ZoneTransferAccept(**EXAMPLE_ACCEPT)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE_ACCEPT['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE_ACCEPT['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE_ACCEPT['key'], sot.key)
        self.assertEqual(EXAMPLE_ACCEPT['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE_ACCEPT['status'], sot.status)
        self.assertEqual(EXAMPLE_ACCEPT['zone_id'], sot.zone_id)
        self.assertEqual(EXAMPLE_ACCEPT['zone_transfer_request_id'],
                         sot.zone_transfer_request_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import connection
from openstack.tests.functional import base

TEST_IMAGE_NAME = 'Test Image'


class TestImage(base.BaseFunctionalTest):

    class ImageOpts:
        def __init__(self):
            self.image_api_version = '2'

    def setUp(self):
        super(TestImage, self).setUp()
        opts = self.ImageOpts()
        self.conn = connection.from_config(
            cloud_name=base.TEST_CLOUD_NAME, options=opts)

        self.img = self.conn.image.create_image(
            name=TEST_IMAGE_NAME,
            disk_format='raw',
            container_format='bare',
            # TODO(mordred): This is not doing what people think it is doing.
            # This is EPICLY broken. However, rather than fixing it as it is,
            # we need to just replace the image upload code with the stuff
            # from shade. Figuring out mapping the crap-tastic arbitrary
            # extra key-value pairs into Resource is going to be fun.
            properties=dict(
                description="This is not an image"
            ),
            data=open('CONTRIBUTING.rst', 'r')
        )
        self.addCleanup(self.conn.image.delete_image, self.img)

    def test_get_image(self):
        img2 = self.conn.image.get_image(self.img)
        self.assertEqual(self.img, img2)

    def test_get_images_schema(self):
        schema = self.conn.image.get_images_schema()
        self.assertIsNotNone(schema)

    def test_get_image_schema(self):
        schema = self.conn.image.get_image_schema()
        self.assertIsNotNone(schema)

    def test_get_members_schema(self):
        schema = self.conn.image.get_members_schema()
        self.assertIsNotNone(schema)

    def test_get_member_schema(self):
        schema = self.conn.image.get_member_schema()
        self.assertIsNotNone(schema)

    def test_list_tasks(self):
        tasks = self.conn.image.tasks()
        self.assertIsNotNone(tasks)

    def test_tags(self):
        img = self.conn.image.get_image(self.img)
        self.conn.image.add_tag(img, 't1')
        self.conn.image.add_tag(img, 't2')
        # Ensure list with array of tags return us our image
        list_img = list(self.conn.image.images(tag=['t1', 't2']))[0]
        self.assertEqual(img.id, list_img.id)
        self.assertIn('t1', list_img.tags)
        self.assertIn('t2', list_img.tags)
        self.conn.image.remove_tag(img, 't1')
        # Refetch img to verify tags
        img = self.conn.image.get_image(self.img)
        self.assertIn('t2', img.tags)
        self.assertNotIn('t1', img.tags)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.block_storage.v3 import backup
from openstack.tests.unit import base


class TestVolumeBackups(base.TestCase):
    def setUp(self):
        super(TestVolumeBackups, self).setUp()
        self.use_cinder()

    def _compare_backups(self, exp, real):
        self.assertDictEqual(
            backup.Backup(**exp).to_dict(computed=False),
            real.to_dict(computed=False))

    def test_search_volume_backups(self):
        name = 'Volume1'
        vol1 = {'name': name, 'availability_zone': 'az1'}
        vol2 = {'name': name, 'availability_zone': 'az1'}
        vol3 = {'name': 'Volume2', 'availability_zone': 'az2'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['backups', 'detail']),
                 json={"backups": [vol1, vol2, vol3]})])
        result = self.cloud.search_volume_backups(
            name, {'availability_zone': 'az1'})
        self.assertEqual(len(result), 2)
        for a, b in zip([vol1, vol2], result):
            self._compare_backups(a, b)
        self.assert_calls()

    def test_get_volume_backup(self):
        name = 'Volume1'
        vol1 = {'name': name, 'availability_zone': 'az1'}
        vol2 = {'name': name, 'availability_zone': 'az2'}
        vol3 = {'name': 'Volume2', 'availability_zone': 'az1'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['backups', 'detail']),
                 json={"backups": [vol1, vol2, vol3]})])
        result = self.cloud.get_volume_backup(
            name, {'availability_zone': 'az1'})
        self._compare_backups(vol1, result)
        self.assert_calls()

    def test_list_volume_backups(self):
        backup = {'id': '6ff16bdf-44d5-4bf9-b0f3-687549c76414',
                  'status': 'available'}
        search_opts = {'status': 'available'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public', append=['backups', 'detail'],
                     qs_elements=['='.join(i) for i in search_opts.items()]),
                 json={"backups": [backup]})])
        result = self.cloud.list_volume_backups(True, search_opts)
        self.assertEqual(len(result), 1)

        self._compare_backups(backup, result[0])
        self.assert_calls()

    def test_delete_volume_backup_wait(self):
        backup_id = '6ff16bdf-44d5-4bf9-b0f3-687549c76414'
        backup = {'id': backup_id, 'status': 'available'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups', 'detail']),
                 json={"backups": [backup]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups', backup_id])),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups', backup_id]),
                 json={"backup": backup}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups', backup_id]),
                 status_code=404)])
        self.cloud.delete_volume_backup(backup_id, False, True, 1)
        self.assert_calls()

    def test_delete_volume_backup_force(self):
        backup_id = '6ff16bdf-44d5-4bf9-b0f3-687549c76414'
        backup = {'id': backup_id, 'status': 'available'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups', 'detail']),
                 json={"backups": [backup]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups', backup_id, 'action']),
                 json={'os-force_delete': {}},
                 validate=dict(json={u'os-force_delete': {}})),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups', backup_id]),
                 json={"backup": backup}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups', backup_id]),
                 status_code=404)
        ])
        self.cloud.delete_volume_backup(backup_id, True, True, 1)
        self.assert_calls()

    def test_create_volume_backup(self):
        volume_id = '1234'
        backup_name = 'bak1'
        bak1 = {
            'id': '5678',
            'volume_id': volume_id,
            'status': 'available',
            'name': backup_name
        }

        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups']),
                 json={'backup': bak1},
                 validate=dict(json={
                     'backup': {
                         'name': backup_name,
                         'volume_id': volume_id,
                         'description': None,
                         'force': False,
                         'snapshot_id': None,
                         'incremental': False
                     }
                 })),
        ])
        self.cloud.create_volume_backup(volume_id, name=backup_name)
        self.assert_calls()

    def test_create_incremental_volume_backup(self):
        volume_id = '1234'
        backup_name = 'bak1'
        bak1 = {
            'id': '5678',
            'volume_id': volume_id,
            'status': 'available',
            'name': backup_name
        }

        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups']),
                 json={'backup': bak1},
                 validate=dict(json={
                     'backup': {
                         'name': backup_name,
                         'volume_id': volume_id,
                         'description': None,
                         'force': False,
                         'snapshot_id': None,
                         'incremental': True
                     }
                 })),
        ])
        self.cloud.create_volume_backup(volume_id, name=backup_name,
                                        incremental=True)
        self.assert_calls()

    def test_create_volume_backup_from_snapshot(self):
        volume_id = '1234'
        backup_name = 'bak1'
        snapshot_id = '5678'
        bak1 = {
            'id': '5678',
            'volume_id': volume_id,
            'status': 'available',
            'name': 'bak1'
        }

        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'volumev3', 'public',
                     append=['backups']),
                 json={'backup': bak1},
                 validate=dict(json={
                     'backup': {
                         'name': backup_name,
                         'volume_id': volume_id,
                         'description': None,
                         'force': False,
                         'snapshot_id': snapshot_id,
                         'incremental': False
                     }
                 })),

        ])
        self.cloud.create_volume_backup(volume_id, name=backup_name,
                                        snapshot_id=snapshot_id)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v2 import schema
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'additionalProperties': {
        'type': 'string'
    },
    'links': [
        {
            'href': '{self}',
            'rel': 'self'
        },
        {
            'href': '{file}',
            'rel': 'enclosure'
        },
        {
            'href': '{schema}',
            'rel': 'describedby'
        }
    ],
    'name': 'image',
    'properties': {
        'architecture': {
            'description': 'Operating system architecture',
            'is_base': False,
            'type': 'string'
        },
        'visibility': {
            'description': 'Scope of image accessibility',
            'enum': [
                'public',
                'private'
            ],
            'type': 'string'
        }
    }
}


class TestSchema(base.TestCase):
    def test_basic(self):
        sot = schema.Schema()
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('/schemas', sot.base_path)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_make_it(self):
        sot = schema.Schema(**EXAMPLE)
        self.assertEqual(EXAMPLE['properties'], sot.properties)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['additionalProperties'],
                         sot.additional_properties)
